# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The QT1 module of niftyfit, which wraps the Multi-Echo T1 fitting methods
in NiftyFit.
"""

from ..base import TraitedSpec, File, traits, CommandLineInputSpec
from .base import NiftyFitCommand
from ..niftyreg.base import get_custom_path


class FitQt1InputSpec(CommandLineInputSpec):
    """ Input Spec for FitQt1. """
    desc = 'Filename of the 4D Multi-Echo T1 source image.'
    source_file = File(
        position=1,
        exists=True,
        desc=desc,
        argstr='-source %s',
        mandatory=True)

    # Output options:
    t1map_file = File(
        name_source=['source_file'],
        name_template='%s_t1map.nii.gz',
        argstr='-t1map %s',
        desc='Filename of the estimated output T1 map (in ms).')
    m0map_file = File(
        name_source=['source_file'],
        name_template='%s_m0map.nii.gz',
        argstr='-m0map %s',
        desc='Filename of the estimated input M0 map.')
    desc = 'Filename of the estimated output multi-parameter map.'
    mcmap_file = File(
        name_source=['source_file'],
        name_template='%s_mcmap.nii.gz',
        argstr='-mcmap %s',
        desc=desc)
    comp_file = File(
        name_source=['source_file'],
        name_template='%s_comp.nii.gz',
        argstr='-comp %s',
        desc='Filename of the estimated multi-component T1 map.')
    desc = 'Filename of the error map (symmetric matrix, [Diag,OffDiag]).'
    error_file = File(
        name_source=['source_file'],
        name_template='%s_error.nii.gz',
        argstr='-error %s',
        desc=desc)
    syn_file = File(
        name_source=['source_file'],
        name_template='%s_syn.nii.gz',
        argstr='-syn %s',
        desc='Filename of the synthetic ASL data.')
    res_file = File(
        name_source=['source_file'],
        name_template='%s_res.nii.gz',
        argstr='-res %s',
        desc='Filename of the model fit residuals')

    # Other options:
    mask = File(
        position=2,
        exists=True,
        desc='Filename of image mask.',
        argstr='-mask %s')
    prior = File(
        position=3,
        exists=True,
        desc='Filename of parameter prior.',
        argstr='-prior %s')
    te_value = traits.Float(
        desc='TE Echo Time [0ms!].', argstr='-TE %f', position=4)
    tr_value = traits.Float(
        desc='TR Repetition Time [10s!].', argstr='-TR %f', position=5)
    desc = 'Number of components to fit [1] (currently IR/SR only)'
    # set position to be ahead of TIs
    nb_comp = traits.Int(desc=desc, position=6, argstr='-nc %d')
    desc = 'Set LM parameters (initial value, decrease rate) [100,1.2].'
    lm_val = traits.Tuple(
        traits.Float, traits.Float, desc=desc, argstr='-lm %f %f', position=7)
    desc = 'Use Gauss-Newton algorithm [Levenberg-Marquardt].'
    gn_flag = traits.Bool(desc=desc, argstr='-gn', position=8)
    slice_no = traits.Int(
        desc='Fit to single slice number.', argstr='-slice %d', position=9)
    voxel = traits.Tuple(
        traits.Int,
        traits.Int,
        traits.Int,
        desc='Fit to single voxel only.',
        argstr='-voxel %d %d %d',
        position=10)
    maxit = traits.Int(
        desc='NLSQR iterations [100].', argstr='-maxit %d', position=11)

    # IR options:
    sr_flag = traits.Bool(
        desc='Saturation Recovery fitting [default].',
        argstr='-SR',
        position=12)
    ir_flag = traits.Bool(
        desc='Inversion Recovery fitting [default].',
        argstr='-IR',
        position=13)
    tis = traits.List(
        traits.Float,
        position=14,
        desc='Inversion times for T1 data [1s,2s,5s].',
        argstr='-TIs %s',
        sep=' ')
    tis_list = File(
        exists=True,
        argstr='-TIlist %s',
        desc='Filename of list of pre-defined TIs.')
    t1_list = File(
        exists=True,
        argstr='-T1list %s',
        desc='Filename of list of pre-defined T1s')
    t1min = traits.Float(
        desc='Minimum tissue T1 value [400ms].', argstr='-T1min %f')
    t1max = traits.Float(
        desc='Maximum tissue T1 value [4000ms].', argstr='-T1max %f')

    # SPGR options
    spgr = traits.Bool(desc='Spoiled Gradient Echo fitting', argstr='-SPGR')
    flips = traits.List(
        traits.Float, desc='Flip angles', argstr='-flips %s', sep=' ')
    desc = 'Filename of list of pre-defined flip angles (deg).'
    flips_list = File(exists=True, argstr='-fliplist %s', desc=desc)
    desc = 'Filename of B1 estimate for fitting (or include in prior).'
    b1map = File(exists=True, argstr='-b1map %s', desc=desc)

    # MCMC options:
    mcout = File(
        exists=True,
        desc='Filename of mc samples (ascii text file)',
        argstr='-mcout %s')
    mcsamples = traits.Int(
        desc='Number of samples to keep [100].', argstr='-mcsamples %d')
    mcmaxit = traits.Int(
        desc='Number of iterations to run [10,000].', argstr='-mcmaxit %d')
    acceptance = traits.Float(
        desc='Fraction of iterations to accept [0.23].',
        argstr='-acceptance %f')


class FitQt1OutputSpec(TraitedSpec):
    """ Output Spec for FitQt1. """
    t1map_file = File(desc='Filename of the estimated output T1 map (in ms)')
    m0map_file = File(desc='Filename of the m0 map')
    desc = 'Filename of the estimated output multi-parameter map'
    mcmap_file = File(desc=desc)
    comp_file = File(desc='Filename of the estimated multi-component T1 map.')
    desc = 'Filename of the error map (symmetric matrix, [Diag,OffDiag])'
    error_file = File(desc=desc)
    syn_file = File(desc='Filename of the synthetic ASL data')
    res_file = File(desc='Filename of the model fit residuals')


class FitQt1(NiftyFitCommand):
    """Interface for executable fit_qt1 from Niftyfit platform.

    Use NiftyFit to perform Qt1 fitting.

    T1 Fitting Routine (To inversion recovery or spgr data).
    Fits single component T1 maps in the first instance.

    `Source code <https://cmiclab.cs.ucl.ac.uk/CMIC/NiftyFit-Release>`_

    Examples
    --------

    >>> from nipype.interfaces.niftyfit import FitQt1
    >>> fit_qt1 = FitQt1()
    >>> fit_qt1.inputs.source_file = 'TI4D.nii.gz'
    >>> fit_qt1.cmdline
    'fit_qt1 -source TI4D.nii.gz -comp TI4D_comp.nii.gz \
-error TI4D_error.nii.gz -m0map TI4D_m0map.nii.gz -mcmap TI4D_mcmap.nii.gz \
-res TI4D_res.nii.gz -syn TI4D_syn.nii.gz -t1map TI4D_t1map.nii.gz'

    """
    _cmd = get_custom_path('fit_qt1', env_dir='NIFTYFITDIR')
    input_spec = FitQt1InputSpec
    output_spec = FitQt1OutputSpec
    _suffix = '_fit_qt1'



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .specialized import (
    BRAINSCut, BRAINSROIAuto, BRAINSConstellationDetector,
    BRAINSCreateLabelMapFromProbabilityMaps, BinaryMaskEditorBasedOnLandmarks,
    BRAINSMultiSTAPLE, BRAINSABC, ESLR)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Tests for the engine module
"""
import numpy as np
import scipy.sparse as ssp
import re

import mock

from nipype.pipeline.plugins.tools import report_crash


def test_report_crash():
    with mock.patch('pickle.dump', mock.MagicMock()) as mock_pickle_dump:
        with mock.patch('nipype.pipeline.plugins.tools.format_exception',
                        mock.MagicMock()):  # see iss 1517
            mock_pickle_dump.return_value = True
            mock_node = mock.MagicMock(name='mock_node')
            mock_node._id = 'an_id'
            mock_node.config = {
                'execution': {
                    'crashdump_dir': '.',
                    'crashfile_format': 'pklz',
                }
            }

            actual_crashfile = report_crash(mock_node)

            expected_crashfile = re.compile(
                '.*/crash-.*-an_id-[0-9a-f\-]*.pklz')

            assert expected_crashfile.match(
                actual_crashfile).group() == actual_crashfile
            assert mock_pickle_dump.call_count == 1


'''
Can use the following code to test that a mapnode crash continues successfully
Need to put this into a unit-test with a timeout

import nipype.interfaces.utility as niu
import nipype.pipeline.engine as pe

wf = pe.Workflow(name='test')

def func(arg1):
    if arg1 == 2:
        raise Exception('arg cannot be ' + str(arg1))
    return arg1

funkynode = pe.MapNode(niu.Function(function=func, input_names=['arg1'],
                                                   output_names=['out']),
                       iterfield=['arg1'],
                       name = 'functor')
funkynode.inputs.arg1 = [1,2]

wf.add_nodes([funkynode])
wf.base_dir = '/tmp'

wf.run(plugin='MultiProc')
'''



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class ResampleDTIVolumeInputSpec(CommandLineInputSpec):
    inputVolume = File(
        position=-2,
        desc="Input volume to be resampled",
        exists=True,
        argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Resampled Volume",
        argstr="%s")
    Reference = File(
        desc="Reference Volume (spacing,size,orientation,origin)",
        exists=True,
        argstr="--Reference %s")
    transformationFile = File(exists=True, argstr="--transformationFile %s")
    defField = File(
        desc=
        "File containing the deformation field (3D vector image containing vectors with 3 components)",
        exists=True,
        argstr="--defField %s")
    hfieldtype = traits.Enum(
        "displacement",
        "h-Field",
        desc="Set if the deformation field is an -Field",
        argstr="--hfieldtype %s")
    interpolation = traits.Enum(
        "linear",
        "nn",
        "ws",
        "bs",
        desc=
        "Sampling algorithm (linear , nn (nearest neighborhoor), ws (WindowedSinc), bs (BSpline) )",
        argstr="--interpolation %s")
    correction = traits.Enum(
        "zero",
        "none",
        "abs",
        "nearest",
        desc=
        "Correct the tensors if computed tensor is not semi-definite positive",
        argstr="--correction %s")
    transform_tensor_method = traits.Enum(
        "PPD",
        "FS",
        desc=
        "Chooses between 2 methods to transform the tensors: Finite Strain (FS), faster but less accurate, or Preservation of the Principal Direction (PPD)",
        argstr="--transform_tensor_method %s")
    transform_order = traits.Enum(
        "input-to-output",
        "output-to-input",
        desc="Select in what order the transforms are read",
        argstr="--transform_order %s")
    notbulk = traits.Bool(
        desc=
        "The transform following the BSpline transform is not set as a bulk transform for the BSpline transform",
        argstr="--notbulk ")
    spaceChange = traits.Bool(
        desc=
        "Space Orientation between transform and image is different (RAS/LPS) (warning: if the transform is a Transform Node in Slicer3, do not select)",
        argstr="--spaceChange ")
    rotation_point = traits.List(
        desc="Center of rotation (only for rigid and affine transforms)",
        argstr="--rotation_point %s")
    centered_transform = traits.Bool(
        desc=
        "Set the center of the transformation to the center of the input image (only for rigid and affine transforms)",
        argstr="--centered_transform ")
    image_center = traits.Enum(
        "input",
        "output",
        desc=
        "Image to use to center the transform (used only if \'Centered Transform\' is selected)",
        argstr="--image_center %s")
    Inverse_ITK_Transformation = traits.Bool(
        desc=
        "Inverse the transformation before applying it from output image to input image (only for rigid and affine transforms)",
        argstr="--Inverse_ITK_Transformation ")
    spacing = InputMultiPath(
        traits.Float,
        desc="Spacing along each dimension (0 means use input spacing)",
        sep=",",
        argstr="--spacing %s")
    size = InputMultiPath(
        traits.Float,
        desc="Size along each dimension (0 means use input size)",
        sep=",",
        argstr="--size %s")
    origin = traits.List(
        desc="Origin of the output Image", argstr="--origin %s")
    direction_matrix = InputMultiPath(
        traits.Float,
        desc=
        "9 parameters of the direction matrix by rows (ijk to LPS if LPS transform, ijk to RAS if RAS transform)",
        sep=",",
        argstr="--direction_matrix %s")
    number_of_thread = traits.Int(
        desc="Number of thread used to compute the output image",
        argstr="--number_of_thread %d")
    default_pixel_value = traits.Float(
        desc=
        "Default pixel value for samples falling outside of the input region",
        argstr="--default_pixel_value %f")
    window_function = traits.Enum(
        "h",
        "c",
        "w",
        "l",
        "b",
        desc=
        "Window Function , h = Hamming , c = Cosine , w = Welch , l = Lanczos , b = Blackman",
        argstr="--window_function %s")
    spline_order = traits.Int(
        desc="Spline Order (Spline order may be from 0 to 5)",
        argstr="--spline_order %d")
    transform_matrix = InputMultiPath(
        traits.Float,
        desc=
        "12 parameters of the transform matrix by rows ( --last 3 being translation-- )",
        sep=",",
        argstr="--transform_matrix %s")
    transform = traits.Enum(
        "rt",
        "a",
        desc="Transform algorithm, rt = Rigid Transform, a = Affine Transform",
        argstr="--transform %s")


class ResampleDTIVolumeOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Resampled Volume", exists=True)


class ResampleDTIVolume(SEMLikeCommandLine):
    """title: Resample DTI Volume

category: Diffusion.Diffusion Tensor Images

description: Resampling an image is a very important task in image analysis. It is especially important in the frame of image registration. This module implements DT image resampling through the use of itk Transforms. The resampling is controlled by the Output Spacing. "Resampling" is performed in space coordinates, not pixel/grid coordinates. It is quite important to ensure that image spacing is properly set on the images involved. The interpolator is required since the mapping from one space to the other will often require evaluation of the intensity of the image at non-grid positions.

version: 0.1

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/ResampleDTI

contributor: Francois Budin (UNC)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149. Information on the National Centers for Biomedical Computing can be obtained from http://nihroadmap.nih.gov/bioinformatics

"""

    input_spec = ResampleDTIVolumeInputSpec
    output_spec = ResampleDTIVolumeOutputSpec
    _cmd = "ResampleDTIVolume "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}


class DWIRicianLMMSEFilterInputSpec(CommandLineInputSpec):
    iter = traits.Int(
        desc="Number of iterations for the noise removal filter.",
        argstr="--iter %d")
    re = InputMultiPath(
        traits.Int, desc="Estimation radius.", sep=",", argstr="--re %s")
    rf = InputMultiPath(
        traits.Int, desc="Filtering radius.", sep=",", argstr="--rf %s")
    mnvf = traits.Int(
        desc="Minimum number of voxels in kernel used for filtering.",
        argstr="--mnvf %d")
    mnve = traits.Int(
        desc="Minimum number of voxels in kernel used for estimation.",
        argstr="--mnve %d")
    minnstd = traits.Int(
        desc="Minimum allowed noise standard deviation.",
        argstr="--minnstd %d")
    maxnstd = traits.Int(
        desc="Maximum allowed noise standard deviation.",
        argstr="--maxnstd %d")
    hrf = traits.Float(
        desc="How many histogram bins per unit interval.", argstr="--hrf %f")
    uav = traits.Bool(
        desc="Use absolute value in case of negative square.", argstr="--uav ")
    inputVolume = File(
        position=-2, desc="Input DWI volume.", exists=True, argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output DWI volume.",
        argstr="%s")
    compressOutput = traits.Bool(
        desc="Compress the data of the compressed file using gzip",
        argstr="--compressOutput ")


class DWIRicianLMMSEFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output DWI volume.", exists=True)


class DWIRicianLMMSEFilter(SEMLikeCommandLine):
    """title: DWI Rician LMMSE Filter

category: Diffusion.Diffusion Weighted Images

description: This module reduces noise (or unwanted detail) on a set of diffusion weighted images. For this, it filters the image in the mean squared error sense using a Rician noise model. Images corresponding to each gradient direction, including baseline, are processed individually. The noise parameter is automatically estimated (noise estimation improved but slower).
Note that this is a general purpose filter for MRi images. The module jointLMMSE has been specifically designed for DWI volumes and shows a better performance, so its use is recommended instead.
A complete description of the algorithm in this module can be found in:
S. Aja-Fernandez, M. Niethammer, M. Kubicki, M. Shenton, and C.-F. Westin. Restoration of DWI data using a Rician LMMSE estimator. IEEE Transactions on Medical Imaging, 27(10): pp. 1389-1403, Oct. 2008.

version: 0.1.1.$Revision: 1 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/RicianLMMSEImageFilter

contributor: Antonio Tristan Vega (UVa), Santiago Aja Fernandez (UVa), Marc Niethammer (UNC)

acknowledgements: Partially founded by grant number TEC2007-67073/TCM from the Comision Interministerial de Ciencia y Tecnologia (Spain).

"""

    input_spec = DWIRicianLMMSEFilterInputSpec
    output_spec = DWIRicianLMMSEFilterOutputSpec
    _cmd = "DWIRicianLMMSEFilter "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}


class TractographyLabelMapSeedingInputSpec(CommandLineInputSpec):
    InputVolume = File(
        position=-2, desc="Input DTI volume", exists=True, argstr="%s")
    inputroi = File(
        desc="Label map with seeding ROIs",
        exists=True,
        argstr="--inputroi %s")
    OutputFibers = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Tractography result",
        argstr="%s")
    useindexspace = traits.Bool(
        desc="Seed at IJK voxel grid", argstr="--useindexspace ")
    seedspacing = traits.Float(
        desc=
        "Spacing (in mm) between seed points, only matters if use Use Index Space is off",
        argstr="--seedspacing %f")
    randomgrid = traits.Bool(
        desc="Enable random placing of seeds", argstr="--randomgrid ")
    clthreshold = traits.Float(
        desc="Minimum Linear Measure for the seeding to start.",
        argstr="--clthreshold %f")
    minimumlength = traits.Float(
        desc="Minimum length of the fibers (in mm)",
        argstr="--minimumlength %f")
    maximumlength = traits.Float(
        desc="Maximum length of fibers (in mm)", argstr="--maximumlength %f")
    stoppingmode = traits.Enum(
        "LinearMeasure",
        "FractionalAnisotropy",
        desc="Tensor measurement used to stop the tractography",
        argstr="--stoppingmode %s")
    stoppingvalue = traits.Float(
        desc=
        "Tractography will stop when the stopping measurement drops below this value",
        argstr="--stoppingvalue %f")
    stoppingcurvature = traits.Float(
        desc=
        "Tractography will stop if radius of curvature becomes smaller than this number units are degrees per mm",
        argstr="--stoppingcurvature %f")
    integrationsteplength = traits.Float(
        desc="Distance between points on the same fiber in mm",
        argstr="--integrationsteplength %f")
    label = traits.Int(
        desc="Label value that defines seeding region.", argstr="--label %d")
    writetofile = traits.Bool(
        desc="Write fibers to disk or create in the scene?",
        argstr="--writetofile ")
    outputdirectory = traits.Either(
        traits.Bool,
        Directory(),
        hash_files=False,
        desc="Directory in which to save fiber(s)",
        argstr="--outputdirectory %s")
    name = traits.Str(desc="Name to use for fiber files", argstr="--name %s")


class TractographyLabelMapSeedingOutputSpec(TraitedSpec):
    OutputFibers = File(position=-1, desc="Tractography result", exists=True)
    outputdirectory = Directory(
        desc="Directory in which to save fiber(s)", exists=True)


class TractographyLabelMapSeeding(SEMLikeCommandLine):
    """title: Tractography Label Map Seeding

category: Diffusion.Diffusion Tensor Images

description: Seed tracts on a Diffusion Tensor Image (DT) from a label map

version: 0.1.0.$Revision: 1892 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/Seeding

license: slicer3

contributor: Raul San Jose (SPL, BWH), Demian Wassermann (SPL, BWH)

acknowledgements: Laboratory of Mathematics in Imaging. This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = TractographyLabelMapSeedingInputSpec
    output_spec = TractographyLabelMapSeedingOutputSpec
    _cmd = "TractographyLabelMapSeeding "
    _outputs_filenames = {
        'OutputFibers': 'OutputFibers.vtk',
        'outputdirectory': 'outputdirectory'
    }


class DWIJointRicianLMMSEFilterInputSpec(CommandLineInputSpec):
    re = InputMultiPath(
        traits.Int, desc="Estimation radius.", sep=",", argstr="--re %s")
    rf = InputMultiPath(
        traits.Int, desc="Filtering radius.", sep=",", argstr="--rf %s")
    ng = traits.Int(
        desc=
        "The number of the closest gradients that are used to jointly filter a given gradient direction (0 to use all).",
        argstr="--ng %d")
    inputVolume = File(
        position=-2, desc="Input DWI volume.", exists=True, argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output DWI volume.",
        argstr="%s")
    compressOutput = traits.Bool(
        desc="Compress the data of the compressed file using gzip",
        argstr="--compressOutput ")


class DWIJointRicianLMMSEFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output DWI volume.", exists=True)


class DWIJointRicianLMMSEFilter(SEMLikeCommandLine):
    """title: DWI Joint Rician LMMSE Filter

category: Diffusion.Diffusion Weighted Images

description: This module reduces Rician noise (or unwanted detail) on a set of diffusion weighted images. For this, it filters the image in the mean squared error sense using a Rician noise model. The N closest gradient directions to the direction being processed are filtered together to improve the results: the noise-free signal is seen as an n-diemensional vector which has to be estimated with the LMMSE method from a set of corrupted measurements. To that end, the covariance matrix of the noise-free vector and the cross covariance between this signal and the noise have to be estimated, which is done taking into account the image formation process.
The noise parameter is automatically estimated from a rough segmentation of the background of the image. In this area the signal is simply 0, so that Rician statistics reduce to Rayleigh and the noise power can be easily estimated from the mode of the histogram.
A complete description of the algorithm may be found in:
Antonio Tristan-Vega and Santiago Aja-Fernandez, DWI filtering using joint information for DTI and HARDI, Medical Image Analysis, Volume 14, Issue 2, Pages 205-218. 2010.

version: 0.1.1.$Revision: 1 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/JointRicianLMMSEImageFilter

contributor: Antonio Tristan Vega (UVa), Santiago Aja Fernandez (UVa)

acknowledgements: Partially founded by grant number TEC2007-67073/TCM from the Comision Interministerial de Ciencia y Tecnologia (Spain).

"""

    input_spec = DWIJointRicianLMMSEFilterInputSpec
    output_spec = DWIJointRicianLMMSEFilterOutputSpec
    _cmd = "DWIJointRicianLMMSEFilter "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}


class DiffusionWeightedVolumeMaskingInputSpec(CommandLineInputSpec):
    inputVolume = File(
        position=-4, desc="Input DWI volume", exists=True, argstr="%s")
    outputBaseline = traits.Either(
        traits.Bool,
        File(),
        position=-2,
        hash_files=False,
        desc="Estimated baseline volume",
        argstr="%s")
    thresholdMask = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Otsu Threshold Mask",
        argstr="%s")
    otsuomegathreshold = traits.Float(
        desc=
        "Control the sharpness of the threshold in the Otsu computation. 0: lower threshold, 1: higher threhold",
        argstr="--otsuomegathreshold %f")
    removeislands = traits.Bool(
        desc="Remove Islands in Threshold Mask?", argstr="--removeislands ")


class DiffusionWeightedVolumeMaskingOutputSpec(TraitedSpec):
    outputBaseline = File(
        position=-2, desc="Estimated baseline volume", exists=True)
    thresholdMask = File(position=-1, desc="Otsu Threshold Mask", exists=True)


class DiffusionWeightedVolumeMasking(SEMLikeCommandLine):
    """title: Diffusion Weighted Volume Masking

category: Diffusion.Diffusion Weighted Images

description: <p>Performs a mask calculation from a diffusion weighted (DW) image.</p><p>Starting from a dw image, this module computes the baseline image averaging all the images without diffusion weighting and then applies the otsu segmentation algorithm in order to produce a mask. this mask can then be used when estimating the diffusion tensor (dt) image, not to estimate tensors all over the volume.</p>

version: 0.1.0.$Revision: 1892 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/DiffusionWeightedMasking

license: slicer3

contributor: Demian Wassermann (SPL, BWH)

"""

    input_spec = DiffusionWeightedVolumeMaskingInputSpec
    output_spec = DiffusionWeightedVolumeMaskingOutputSpec
    _cmd = "DiffusionWeightedVolumeMasking "
    _outputs_filenames = {
        'outputBaseline': 'outputBaseline.nii',
        'thresholdMask': 'thresholdMask.nii'
    }


class DTIimportInputSpec(CommandLineInputSpec):
    inputFile = File(
        position=-2, desc="Input DTI file", exists=True, argstr="%s")
    outputTensor = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output DTI volume",
        argstr="%s")
    testingmode = traits.Bool(
        desc=
        "Enable testing mode. Sample helix file (helix-DTI.nhdr) will be loaded into Slicer and converted in Nifti.",
        argstr="--testingmode ")


class DTIimportOutputSpec(TraitedSpec):
    outputTensor = File(position=-1, desc="Output DTI volume", exists=True)


class DTIimport(SEMLikeCommandLine):
    """title: DTIimport

category: Diffusion.Diffusion Data Conversion

description: Import tensor datasets from various formats, including the NifTi file format

version: 1.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/DTIImport

contributor: Sonia Pujol (SPL, BWH)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NA-MIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = DTIimportInputSpec
    output_spec = DTIimportOutputSpec
    _cmd = "DTIimport "
    _outputs_filenames = {'outputTensor': 'outputTensor.nii'}


class DWIToDTIEstimationInputSpec(CommandLineInputSpec):
    inputVolume = File(
        position=-3, desc="Input DWI volume", exists=True, argstr="%s")
    mask = File(
        desc="Mask where the tensors will be computed",
        exists=True,
        argstr="--mask %s")
    outputTensor = traits.Either(
        traits.Bool,
        File(),
        position=-2,
        hash_files=False,
        desc="Estimated DTI volume",
        argstr="%s")
    outputBaseline = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Estimated baseline volume",
        argstr="%s")
    enumeration = traits.Enum(
        "LS",
        "WLS",
        desc="LS: Least Squares, WLS: Weighted Least Squares",
        argstr="--enumeration %s")
    shiftNeg = traits.Bool(
        desc=
        "Shift eigenvalues so all are positive (accounts for bad tensors related to noise or acquisition error)",
        argstr="--shiftNeg ")


class DWIToDTIEstimationOutputSpec(TraitedSpec):
    outputTensor = File(position=-2, desc="Estimated DTI volume", exists=True)
    outputBaseline = File(
        position=-1, desc="Estimated baseline volume", exists=True)


class DWIToDTIEstimation(SEMLikeCommandLine):
    """title: DWI to DTI Estimation

category: Diffusion.Diffusion Weighted Images

description: Performs a tensor model estimation from diffusion weighted images.

There are three estimation methods available: least squares, weigthed least squares and non-linear estimation. The first method is the traditional method for tensor estimation and the fastest one. Weighted least squares takes into account the noise characteristics of the MRI images to weight the DWI samples used in the estimation based on its intensity magnitude. The last method is the more complex.

version: 0.1.0.$Revision: 1892 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/DiffusionTensorEstimation

license: slicer3

contributor: Raul San Jose (SPL, BWH)

acknowledgements: This command module is based on the estimation functionality provided by the Teem library. This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = DWIToDTIEstimationInputSpec
    output_spec = DWIToDTIEstimationOutputSpec
    _cmd = "DWIToDTIEstimation "
    _outputs_filenames = {
        'outputTensor': 'outputTensor.nii',
        'outputBaseline': 'outputBaseline.nii'
    }


class DiffusionTensorScalarMeasurementsInputSpec(CommandLineInputSpec):
    inputVolume = File(
        position=-3, desc="Input DTI volume", exists=True, argstr="%s")
    outputScalar = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Scalar volume derived from tensor",
        argstr="%s")
    enumeration = traits.Enum(
        "Trace",
        "Determinant",
        "RelativeAnisotropy",
        "FractionalAnisotropy",
        "Mode",
        "LinearMeasure",
        "PlanarMeasure",
        "SphericalMeasure",
        "MinEigenvalue",
        "MidEigenvalue",
        "MaxEigenvalue",
        "MaxEigenvalueProjectionX",
        "MaxEigenvalueProjectionY",
        "MaxEigenvalueProjectionZ",
        "RAIMaxEigenvecX",
        "RAIMaxEigenvecY",
        "RAIMaxEigenvecZ",
        "MaxEigenvecX",
        "MaxEigenvecY",
        "MaxEigenvecZ",
        "D11",
        "D22",
        "D33",
        "ParallelDiffusivity",
        "PerpendicularDffusivity",
        desc="An enumeration of strings",
        argstr="--enumeration %s")


class DiffusionTensorScalarMeasurementsOutputSpec(TraitedSpec):
    outputScalar = File(
        position=-1, desc="Scalar volume derived from tensor", exists=True)


class DiffusionTensorScalarMeasurements(SEMLikeCommandLine):
    """title: Diffusion Tensor Scalar Measurements

category: Diffusion.Diffusion Tensor Images

description: Compute a set of different scalar measurements from a tensor field, specially oriented for Diffusion Tensors where some rotationally invariant measurements, like Fractional Anisotropy, are highly used to describe the anistropic behaviour of the tensor.

version: 0.1.0.$Revision: 1892 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/DiffusionTensorMathematics

contributor: Raul San Jose (SPL, BWH)

acknowledgements: LMI

"""

    input_spec = DiffusionTensorScalarMeasurementsInputSpec
    output_spec = DiffusionTensorScalarMeasurementsOutputSpec
    _cmd = "DiffusionTensorScalarMeasurements "
    _outputs_filenames = {'outputScalar': 'outputScalar.nii'}


class DTIexportInputSpec(CommandLineInputSpec):
    inputTensor = File(
        position=-2, desc="Input DTI volume", exists=True, argstr="%s")
    outputFile = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output DTI file",
        argstr="%s")


class DTIexportOutputSpec(TraitedSpec):
    outputFile = File(position=-1, desc="Output DTI file", exists=True)


class DTIexport(SEMLikeCommandLine):
    """title: DTIexport

category: Diffusion.Diffusion Data Conversion

description: Export DTI data to various file formats

version: 1.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/DTIExport

contributor: Sonia Pujol (SPL, BWH)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NA-MIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = DTIexportInputSpec
    output_spec = DTIexportOutputSpec
    _cmd = "DTIexport "
    _outputs_filenames = {'outputFile': 'outputFile'}



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The :py:mod:`nipype.interfaces.elastix` provides the interface to
the elastix registration software.

.. note:: http://elastix.isi.uu.nl/


"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from ... import logging
from ..base import CommandLineInputSpec, Directory, traits
iflogger = logging.getLogger('nipype.interface')


class ElastixBaseInputSpec(CommandLineInputSpec):
    output_path = Directory(
        './',
        exists=True,
        mandatory=True,
        usedefault=True,
        argstr='-out %s',
        desc='output directory')
    num_threads = traits.Int(
        1, usedefault=True,
        argstr='-threads %01d',
        nohash=True,
        desc='set the maximum number of threads of elastix')



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import MRISPreproc


def test_MRISPreproc_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fsgd_file=dict(
            argstr='--fsgd %s',
            extensions=None,
            xor=('subjects', 'fsgd_file', 'subject_file'),
        ),
        fwhm=dict(
            argstr='--fwhm %f',
            xor=['num_iters'],
        ),
        fwhm_source=dict(
            argstr='--fwhm-src %f',
            xor=['num_iters_source'],
        ),
        hemi=dict(
            argstr='--hemi %s',
            mandatory=True,
        ),
        num_iters=dict(
            argstr='--niters %d',
            xor=['fwhm'],
        ),
        num_iters_source=dict(
            argstr='--niterssrc %d',
            xor=['fwhm_source'],
        ),
        out_file=dict(
            argstr='--out %s',
            extensions=None,
            genfile=True,
        ),
        proj_frac=dict(argstr='--projfrac %s', ),
        smooth_cortex_only=dict(argstr='--smooth-cortex-only', ),
        source_format=dict(argstr='--srcfmt %s', ),
        subject_file=dict(
            argstr='--f %s',
            extensions=None,
            xor=('subjects', 'fsgd_file', 'subject_file'),
        ),
        subjects=dict(
            argstr='--s %s...',
            xor=('subjects', 'fsgd_file', 'subject_file'),
        ),
        subjects_dir=dict(),
        surf_area=dict(
            argstr='--area %s',
            xor=('surf_measure', 'surf_measure_file', 'surf_area'),
        ),
        surf_dir=dict(argstr='--surfdir %s', ),
        surf_measure=dict(
            argstr='--meas %s',
            xor=('surf_measure', 'surf_measure_file', 'surf_area'),
        ),
        surf_measure_file=dict(
            argstr='--is %s...',
            xor=('surf_measure', 'surf_measure_file', 'surf_area'),
        ),
        target=dict(
            argstr='--target %s',
            mandatory=True,
        ),
        vol_measure_file=dict(argstr='--iv %s %s...', ),
    )
    inputs = MRISPreproc.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MRISPreproc_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = MRISPreproc.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..tracking import FilterTracks


def test_FilterTracks_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        debug=dict(
            argstr='-debug',
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        exclude_file=dict(
            argstr='-exclude %s',
            extensions=None,
            xor=['exclude_file', 'exclude_spec'],
        ),
        exclude_spec=dict(
            argstr='-exclude %s',
            position=2,
            sep=',',
            units='mm',
            xor=['exclude_file', 'exclude_spec'],
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        include_file=dict(
            argstr='-include %s',
            extensions=None,
            xor=['include_file', 'include_spec'],
        ),
        include_spec=dict(
            argstr='-include %s',
            position=2,
            sep=',',
            units='mm',
            xor=['include_file', 'include_spec'],
        ),
        invert=dict(argstr='-invert', ),
        minimum_tract_length=dict(
            argstr='-minlength %s',
            units='mm',
        ),
        no_mask_interpolation=dict(argstr='-nomaskinterp', ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            hash_files=False,
            name_source=['in_file'],
            name_template='%s_filt',
            position=-1,
        ),
        quiet=dict(
            argstr='-quiet',
            position=1,
        ),
    )
    inputs = FilterTracks.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FilterTracks_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = FilterTracks.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test the resource management of MultiProc
"""
import os
import pytest
from nipype.pipeline import engine as pe
from nipype.interfaces import base as nib


class InputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(desc='a random int')
    input2 = nib.traits.Int(desc='a random int')


class OutputSpec(nib.TraitedSpec):
    output1 = nib.traits.List(nib.traits.Int, desc='outputs')


class MultiprocTestInterface(nib.BaseInterface):
    input_spec = InputSpec
    output_spec = OutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output1'] = [1, self.inputs.input1]
        return outputs


def test_run_multiproc(tmpdir):
    tmpdir.chdir()

    pipe = pe.Workflow(name='pipe')
    mod1 = pe.Node(MultiprocTestInterface(), name='mod1')
    mod2 = pe.MapNode(
        MultiprocTestInterface(), iterfield=['input1'], name='mod2')
    pipe.connect([(mod1, mod2, [('output1', 'input1')])])
    pipe.base_dir = os.getcwd()
    mod1.inputs.input1 = 1
    pipe.config['execution']['poll_sleep_duration'] = 2
    execgraph = pipe.run(plugin="MultiProc")
    names = [node.fullname for node in execgraph.nodes()]
    node = list(execgraph.nodes())[names.index('pipe.mod1')]
    result = node.get_output('output1')
    assert result == [1, 1]


class InputSpecSingleNode(nib.TraitedSpec):
    input1 = nib.traits.Int(desc='a random int')
    input2 = nib.traits.Int(desc='a random int')


class OutputSpecSingleNode(nib.TraitedSpec):
    output1 = nib.traits.Int(desc='a random int')


class SingleNodeTestInterface(nib.BaseInterface):
    input_spec = InputSpecSingleNode
    output_spec = OutputSpecSingleNode

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output1'] = self.inputs.input1
        return outputs


def test_no_more_memory_than_specified(tmpdir):
    tmpdir.chdir()
    pipe = pe.Workflow(name='pipe')
    n1 = pe.Node(SingleNodeTestInterface(), name='n1', mem_gb=1)
    n2 = pe.Node(SingleNodeTestInterface(), name='n2', mem_gb=1)
    n3 = pe.Node(SingleNodeTestInterface(), name='n3', mem_gb=1)
    n4 = pe.Node(SingleNodeTestInterface(), name='n4', mem_gb=1)

    pipe.connect(n1, 'output1', n2, 'input1')
    pipe.connect(n1, 'output1', n3, 'input1')
    pipe.connect(n2, 'output1', n4, 'input1')
    pipe.connect(n3, 'output1', n4, 'input2')
    n1.inputs.input1 = 1

    max_memory = 0.5
    with pytest.raises(RuntimeError):
        pipe.run(
            plugin='MultiProc',
            plugin_args={
                'memory_gb': max_memory,
                'n_procs': 2
            })


def test_no_more_threads_than_specified(tmpdir):
    tmpdir.chdir()

    pipe = pe.Workflow(name='pipe')
    n1 = pe.Node(SingleNodeTestInterface(), name='n1', n_procs=2)
    n2 = pe.Node(SingleNodeTestInterface(), name='n2', n_procs=2)
    n3 = pe.Node(SingleNodeTestInterface(), name='n3', n_procs=4)
    n4 = pe.Node(SingleNodeTestInterface(), name='n4', n_procs=2)

    pipe.connect(n1, 'output1', n2, 'input1')
    pipe.connect(n1, 'output1', n3, 'input1')
    pipe.connect(n2, 'output1', n4, 'input1')
    pipe.connect(n3, 'output1', n4, 'input2')
    n1.inputs.input1 = 4

    max_threads = 2
    with pytest.raises(RuntimeError):
        pipe.run(plugin='MultiProc', plugin_args={'n_procs': max_threads})


def test_hold_job_until_procs_available(tmpdir):
    tmpdir.chdir()

    pipe = pe.Workflow(name='pipe')
    n1 = pe.Node(SingleNodeTestInterface(), name='n1', n_procs=2)
    n2 = pe.Node(SingleNodeTestInterface(), name='n2', n_procs=2)
    n3 = pe.Node(SingleNodeTestInterface(), name='n3', n_procs=2)
    n4 = pe.Node(SingleNodeTestInterface(), name='n4', n_procs=2)

    pipe.connect(n1, 'output1', n2, 'input1')
    pipe.connect(n1, 'output1', n3, 'input1')
    pipe.connect(n2, 'output1', n4, 'input1')
    pipe.connect(n3, 'output1', n4, 'input2')
    n1.inputs.input1 = 4

    max_threads = 2
    pipe.run(plugin='MultiProc', plugin_args={'n_procs': max_threads})



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import UnpackSDICOMDir


def test_UnpackSDICOMDir_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        config=dict(
            argstr='-cfg %s',
            extensions=None,
            mandatory=True,
            xor=('run_info', 'config', 'seq_config'),
        ),
        dir_structure=dict(argstr='-%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        log_file=dict(
            argstr='-log %s',
            extensions=None,
        ),
        no_info_dump=dict(argstr='-noinfodump', ),
        no_unpack_err=dict(argstr='-no-unpackerr', ),
        output_dir=dict(argstr='-targ %s', ),
        run_info=dict(
            argstr='-run %d %s %s %s',
            mandatory=True,
            xor=('run_info', 'config', 'seq_config'),
        ),
        scan_only=dict(
            argstr='-scanonly %s',
            extensions=None,
        ),
        seq_config=dict(
            argstr='-seqcfg %s',
            extensions=None,
            mandatory=True,
            xor=('run_info', 'config', 'seq_config'),
        ),
        source_dir=dict(
            argstr='-src %s',
            mandatory=True,
        ),
        spm_zeropad=dict(argstr='-nspmzeropad %d', ),
        subjects_dir=dict(),
    )
    inputs = UnpackSDICOMDir.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import GenerateWhiteMatterMask


def test_GenerateWhiteMatterMask_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        binary_mask=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        encoding_file=dict(
            argstr='-grad %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        noise_level_margin=dict(argstr='-margin %s', ),
        out_WMProb_filename=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
    )
    inputs = GenerateWhiteMatterMask.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_GenerateWhiteMatterMask_outputs():
    output_map = dict(WMprobabilitymap=dict(extensions=None, ), )
    outputs = GenerateWhiteMatterMask.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest
from nipype.testing import example_data
from nipype.algorithms.metrics import ErrorMap
import nibabel as nb
import numpy as np
import os


def test_errormap(tmpdir):

    # Single-Spectual
    # Make two fake 2*2*2 voxel volumes
    # John von Neumann's birthday
    volume1 = np.array([[[2.0, 8.0], [1.0, 2.0]], [[1.0, 9.0], [0.0, 3.0]]])
    # Alan Turing's birthday
    volume2 = np.array([[[0.0, 7.0], [2.0, 3.0]], [[1.0, 9.0], [1.0, 2.0]]])
    mask = np.array([[[1, 0], [0, 1]], [[1, 0], [0, 1]]])

    img1 = nb.Nifti1Image(volume1, np.eye(4))
    img2 = nb.Nifti1Image(volume2, np.eye(4))
    maskimg = nb.Nifti1Image(mask, np.eye(4))

    nb.save(img1, tmpdir.join('von.nii.gz').strpath)
    nb.save(img2, tmpdir.join('alan.nii.gz').strpath)
    nb.save(maskimg, tmpdir.join('mask.nii.gz').strpath)

    # Default metric
    errmap = ErrorMap()
    errmap.inputs.in_tst = tmpdir.join('von.nii.gz').strpath
    errmap.inputs.in_ref = tmpdir.join('alan.nii.gz').strpath
    errmap.out_map = tmpdir.join('out_map.nii.gz').strpath
    result = errmap.run()
    assert result.outputs.distance == 1.125

    # Square metric
    errmap.inputs.metric = 'sqeuclidean'
    result = errmap.run()
    assert result.outputs.distance == 1.125

    # Linear metric
    errmap.inputs.metric = 'euclidean'
    result = errmap.run()
    assert result.outputs.distance == 0.875

    # Masked
    errmap.inputs.mask = tmpdir.join('mask.nii.gz').strpath
    result = errmap.run()
    assert result.outputs.distance == 1.0

    # Multi-Spectual
    # Raymond Vahan Damadian's birthday
    volume3 = np.array([[[1.0, 6.0], [0.0, 3.0]], [[1.0, 9.0], [3.0, 6.0]]])

    msvolume1 = np.zeros(shape=(2, 2, 2, 2))
    msvolume1[:, :, :, 0] = volume1
    msvolume1[:, :, :, 1] = volume3
    msimg1 = nb.Nifti1Image(msvolume1, np.eye(4))

    msvolume2 = np.zeros(shape=(2, 2, 2, 2))
    msvolume2[:, :, :, 0] = volume3
    msvolume2[:, :, :, 1] = volume1
    msimg2 = nb.Nifti1Image(msvolume2, np.eye(4))

    nb.save(msimg1, tmpdir.join('von-ray.nii.gz').strpath)
    nb.save(msimg2, tmpdir.join('alan-ray.nii.gz').strpath)

    errmap.inputs.in_tst = tmpdir.join('von-ray.nii.gz').strpath
    errmap.inputs.in_ref = tmpdir.join('alan-ray.nii.gz').strpath
    errmap.inputs.metric = 'sqeuclidean'
    result = errmap.run()
    assert result.outputs.distance == 5.5

    errmap.inputs.metric = 'euclidean'
    result = errmap.run()
    assert result.outputs.distance == np.float32(1.25 * (2**0.5))



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The niftyseg module provides classes for interfacing with `niftyseg
<https://sourceforge.net/projects/niftyseg/>`_ command line tools.
These are the base tools for working with niftyseg.
EM Statistical Segmentation tool is found in niftyseg/em.py
Fill lesions tool is found in niftyseg/lesions.py
Mathematical operation tool is found in niftyseg/maths.py
Patch Match tool is found in niftyseg/patchmatch.py
Statistical operation tool is found in niftyseg/stats.py
Label Fusion and CalcTopNcc tools are in niftyseg/steps.py
Examples
--------
See the docstrings of the individual classes for examples.
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from ..niftyfit.base import NiftyFitCommand


class NiftySegCommand(NiftyFitCommand):
    """
    Base support interface for NiftySeg commands.
    """
    _suffix = '_ns'
    _min_version = None

    def __init__(self, **inputs):
        super(NiftySegCommand, self).__init__(**inputs)

    def get_version(self):
        return super(NiftySegCommand, self).version_from_command(
            cmd='seg_EM', flag='--version')



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import Hemisplit


def test_Hemisplit_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputHemisphereLabelFile=dict(
            argstr='-l %s',
            extensions=None,
            mandatory=True,
        ),
        inputSurfaceFile=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        outputLeftHemisphere=dict(
            argstr='--left %s',
            extensions=None,
            genfile=True,
        ),
        outputLeftPialHemisphere=dict(
            argstr='-pl %s',
            extensions=None,
            genfile=True,
        ),
        outputRightHemisphere=dict(
            argstr='--right %s',
            extensions=None,
            genfile=True,
        ),
        outputRightPialHemisphere=dict(
            argstr='-pr %s',
            extensions=None,
            genfile=True,
        ),
        pialSurfaceFile=dict(
            argstr='-p %s',
            extensions=None,
        ),
        timer=dict(argstr='--timer', ),
        verbosity=dict(argstr='-v %d', ),
    )
    inputs = Hemisplit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Hemisplit_outputs():
    output_map = dict(
        outputLeftHemisphere=dict(extensions=None, ),
        outputLeftPialHemisphere=dict(extensions=None, ),
        outputRightHemisphere=dict(extensions=None, ),
        outputRightPialHemisphere=dict(extensions=None, ),
    )
    outputs = Hemisplit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..matlab import MatlabCommand


def test_MatlabCommand_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        logfile=dict(
            argstr='-logfile %s',
            extensions=None,
        ),
        mfile=dict(usedefault=True, ),
        nodesktop=dict(
            argstr='-nodesktop',
            nohash=True,
            usedefault=True,
        ),
        nosplash=dict(
            argstr='-nosplash',
            nohash=True,
            usedefault=True,
        ),
        paths=dict(),
        postscript=dict(usedefault=True, ),
        prescript=dict(usedefault=True, ),
        script=dict(
            argstr='-r "%s;exit"',
            mandatory=True,
            position=-1,
        ),
        script_file=dict(
            extensions=None,
            usedefault=True,
        ),
        single_comp_thread=dict(
            argstr='-singleCompThread',
            nohash=True,
        ),
        uses_mcr=dict(
            nohash=True,
            xor=['nodesktop', 'nosplash', 'single_comp_thread'],
        ),
    )
    inputs = MatlabCommand.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..diffusion import ResampleDTIVolume


def test_ResampleDTIVolume_inputs():
    input_map = dict(
        Inverse_ITK_Transformation=dict(
            argstr='--Inverse_ITK_Transformation ', ),
        Reference=dict(
            argstr='--Reference %s',
            extensions=None,
        ),
        args=dict(argstr='%s', ),
        centered_transform=dict(argstr='--centered_transform ', ),
        correction=dict(argstr='--correction %s', ),
        defField=dict(
            argstr='--defField %s',
            extensions=None,
        ),
        default_pixel_value=dict(argstr='--default_pixel_value %f', ),
        direction_matrix=dict(
            argstr='--direction_matrix %s',
            sep=',',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hfieldtype=dict(argstr='--hfieldtype %s', ),
        image_center=dict(argstr='--image_center %s', ),
        inputVolume=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        interpolation=dict(argstr='--interpolation %s', ),
        notbulk=dict(argstr='--notbulk ', ),
        number_of_thread=dict(argstr='--number_of_thread %d', ),
        origin=dict(argstr='--origin %s', ),
        outputVolume=dict(
            argstr='%s',
            hash_files=False,
            position=-1,
        ),
        rotation_point=dict(argstr='--rotation_point %s', ),
        size=dict(
            argstr='--size %s',
            sep=',',
        ),
        spaceChange=dict(argstr='--spaceChange ', ),
        spacing=dict(
            argstr='--spacing %s',
            sep=',',
        ),
        spline_order=dict(argstr='--spline_order %d', ),
        transform=dict(argstr='--transform %s', ),
        transform_matrix=dict(
            argstr='--transform_matrix %s',
            sep=',',
        ),
        transform_order=dict(argstr='--transform_order %s', ),
        transform_tensor_method=dict(argstr='--transform_tensor_method %s', ),
        transformationFile=dict(
            argstr='--transformationFile %s',
            extensions=None,
        ),
        window_function=dict(argstr='--window_function %s', ),
    )
    inputs = ResampleDTIVolume.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ResampleDTIVolume_outputs():
    output_map = dict(outputVolume=dict(
        extensions=None,
        position=-1,
    ), )
    outputs = ResampleDTIVolume.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Debug plugin
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import networkx as nx
from .base import PluginBase, logger


class DebugPlugin(PluginBase):
    """Execute workflow in series
    """

    def __init__(self, plugin_args=None):
        super(DebugPlugin, self).__init__(plugin_args=plugin_args)
        if plugin_args and "callable" in plugin_args and \
                hasattr(plugin_args['callable'], '__call__'):
            self._callable = plugin_args['callable']
        else:
            raise ValueError('plugin_args must contain a callable function')

    def run(self, graph, config, updatehash=False):
        """Executes a pre-defined pipeline in a serial order.

        Parameters
        ----------

        graph : networkx digraph
            defines order of execution
        """

        if not isinstance(graph, nx.DiGraph):
            raise ValueError('Input must be a networkx digraph object')
        logger.info("Executing debug plugin")
        for node in nx.topological_sort(graph):
            self._callable(node, graph)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from nipype.interfaces.ants import WarpImageMultiTransform, WarpTimeSeriesImageMultiTransform
import os
import pytest


@pytest.fixture()
def change_dir(request):
    orig_dir = os.getcwd()
    filepath = os.path.dirname(os.path.realpath(__file__))
    datadir = os.path.realpath(os.path.join(filepath, '../../../testing/data'))
    os.chdir(datadir)

    def move2orig():
        os.chdir(orig_dir)

    request.addfinalizer(move2orig)


@pytest.fixture()
def create_wimt():
    wimt = WarpImageMultiTransform()
    wimt.inputs.input_image = 'diffusion_weighted.nii'
    wimt.inputs.reference_image = 'functional.nii'
    wimt.inputs.transformation_series = [
        'func2anat_coreg_Affine.txt', 'func2anat_InverseWarp.nii.gz',
        'dwi2anat_Warp.nii.gz', 'dwi2anat_coreg_Affine.txt'
    ]
    return wimt


def test_WarpImageMultiTransform(change_dir, create_wimt):
    wimt = create_wimt
    assert wimt.cmdline == 'WarpImageMultiTransform 3 diffusion_weighted.nii diffusion_weighted_wimt.nii -R functional.nii \
func2anat_coreg_Affine.txt func2anat_InverseWarp.nii.gz dwi2anat_Warp.nii.gz dwi2anat_coreg_Affine.txt'


def test_WarpImageMultiTransform_invaffine_1(change_dir, create_wimt):
    wimt = create_wimt
    wimt.inputs.invert_affine = [1]
    assert wimt.cmdline == 'WarpImageMultiTransform 3 diffusion_weighted.nii diffusion_weighted_wimt.nii -R functional.nii \
-i func2anat_coreg_Affine.txt func2anat_InverseWarp.nii.gz dwi2anat_Warp.nii.gz dwi2anat_coreg_Affine.txt'


def test_WarpImageMultiTransform_invaffine_2(change_dir, create_wimt):
    wimt = create_wimt
    wimt.inputs.invert_affine = [2]
    assert wimt.cmdline == 'WarpImageMultiTransform 3 diffusion_weighted.nii diffusion_weighted_wimt.nii -R functional.nii func2anat_coreg_Affine.txt func2anat_InverseWarp.nii.gz dwi2anat_Warp.nii.gz -i dwi2anat_coreg_Affine.txt'


def test_WarpImageMultiTransform_invaffine_wrong(change_dir, create_wimt):
    wimt = create_wimt
    wimt.inputs.invert_affine = [3]
    with pytest.raises(Exception):
        assert wimt.cmdline


@pytest.fixture()
def create_wtsimt():
    wtsimt = WarpTimeSeriesImageMultiTransform()
    wtsimt.inputs.input_image = 'resting.nii'
    wtsimt.inputs.reference_image = 'ants_deformed.nii.gz'
    wtsimt.inputs.transformation_series = [
        'ants_Warp.nii.gz', 'ants_Affine.txt'
    ]
    return wtsimt


def test_WarpTimeSeriesImageMultiTransform(change_dir, create_wtsimt):
    wtsimt = create_wtsimt
    assert wtsimt.cmdline == 'WarpTimeSeriesImageMultiTransform 4 resting.nii resting_wtsimt.nii \
-R ants_deformed.nii.gz ants_Warp.nii.gz ants_Affine.txt'


def test_WarpTimeSeriesImageMultiTransform_invaffine(change_dir,
                                                     create_wtsimt):
    wtsimt = create_wtsimt
    wtsimt.inputs.invert_affine = [1]
    assert wtsimt.cmdline == 'WarpTimeSeriesImageMultiTransform 4 resting.nii resting_wtsimt.nii \
-R ants_deformed.nii.gz ants_Warp.nii.gz -i ants_Affine.txt'


def test_WarpTimeSeriesImageMultiTransform_invaffine_wrong(
        change_dir, create_wtsimt):
    wtsimt = create_wtsimt
    wtsimt.inputs.invert_affine = [0]
    with pytest.raises(Exception):
        wtsimt.cmdline
